package com.kundun.graph.controller;

import java.util.ArrayList;
import java.util.HashMap;

import org.apache.log4j.Logger;

import com.kundun.graph.io.Reader;
import com.kundun.graph.io.ReaderFile;
import com.kundun.graph.io.ReaderShellEdge;
import com.kundun.graph.io.ReaderShellGraph;
import com.kundun.graph.io.ReaderShellNode;
import com.kundun.graph.main.Config;
import com.kundun.graph.main.GraphRepository;
import com.kundun.graph.model.Edge;
import com.kundun.graph.model.Graph;
import com.kundun.graph.model.IndirectGraph;
import com.kundun.graph.model.Node;
import com.kundun.graph.parser.MainParser;
import com.kundun.graph.shell.Shell;

public class Add implements Operation {

	private final static Logger LOGGER = Logger.getLogger(Add.class.getName());

	// Whatever graph element - whole graph, node/edge only
	public ArrayList<HashMap<String, String>> graphElements = new ArrayList<HashMap<String, String>>();
	public String graphId;
	
	private String[] input;

	/**
	 * Manage add operation
	 * 
	 * @param input
	 */
	public Add(String[] input) {
		this.input = input;
	}
	
	@Override
	public void doOperation() {
		if (input.length < 3) {
			Shell.invalidSyntax();
		} else {
			switch (input[1].toLowerCase()) {
			case GRAPH:
				if (input.length < 3) {
					Shell.invalidSyntax();
					break;
				}
				switch(input[2].toLowerCase()) {
				case FILE: //read from file
					try {
						MainParser.hashMapToObject(prepareInputForNewGraph(input));
					} catch (Exception e) {
						LOGGER.error(Config.ln() + "Error creating graph from file: ");
						LOGGER.error(e.getMessage());
					} finally {
						
					}
					break;
					
				default: // read from shell
					addGraph();
					break;
				}
				break;
	
			case NODE:
				if (input.length < 4) {
					Shell.invalidSyntax();
					break;
				}
	
				addNode(input[2].trim());
				break;
	
			case EDGE:
				if (input.length < 6) {
					Shell.invalidSyntax();
					break;
				}
	
				addEdge(input[2].trim());
				break;
	
			default:
				Shell.invalidSyntax();
				break;
			}
		}
	}

	/**
	 * Add graph to global storehouse
	 * 
	 * @param graph
	 */
	private void addGraph() {
		ArrayList<HashMap<String, String>> graph = prepareInputForNewGraph(input);
		HashMap<String, String> g = graph.get(0);
		String[] split = g.get("graph").split(SEPARATOR);
		String id = split[0].trim();
		String label = split.length == 2 ? split[1].trim() : "\"null\"";
		Graph newGraph = new IndirectGraph(id, label);
		try {
			if (Graph.getGraphById(newGraph.getId()) != null) {
				Shell.println("Graph with ID '" + newGraph.getId()
						+ "' already exist!");
			} else {
				GraphRepository.globalGraphList.add(newGraph);
				Shell.println("Graph: '" + newGraph.getId()
						+ "' added successfully!");
			}
		} catch (Exception e) {
			Shell.println("Error creating graph... Look into log file for more info.");
			LOGGER.error(Config.ln() + "Graph data error - check input.");
			LOGGER.error(e.getMessage());
		}
	}

	/**
	 * Add node to an existing graph
	 * 
	 * @param graphId
	 * @param node
	 */
	private void addNode(String graphId) {
		ArrayList<HashMap<String, String>> node = prepareInputForNewNode(input);
		HashMap<String, String> n = node.get(0);
		Graph graph = Graph.getGraphById(graphId);

		try {
			String split[] = n.get("node").split(SEPARATOR);
			Node newNode = Node.checkAndCreateUniqueNode(graph.getNodes(),
					split[0].trim(), split[1].trim());
			graph.addNode(newNode);
			Shell.println("Node: '" + newNode.getId()
					+ "' added succesfully to graph '" + graph.getId() + "'!");

			// Change save status of graph
			graph.unsave();
		} catch (Exception e) {
			Shell.println("Error creating node... Look into log file for more info.");
			LOGGER.error(Config.ln() + "Node data error - check input.");
			LOGGER.error(e.getMessage());
		}
	}

	/**
	 * Add edge to an existing graph
	 * 
	 * @param graphId
	 * @param edge
	 */
	private void addEdge(String graphId) {
		ArrayList<HashMap<String, String>> edge = prepareInputForNewEdge(input);
		HashMap<String, String> e = edge.get(0);
		Graph graph = Graph.getGraphById(graphId);

		try {
			String split[] = e.get("edge").split(SEPARATOR);
			Edge newEdge = Edge.checkAndCreateUniqueEdge(graph.getEdges(),
					split[0].trim(),
					Node.getNodeById(graph.getNodes(), split[1].trim()),
					Node.getNodeById(graph.getNodes(), split[2].trim()), split[3].trim());
			graph.addEdge(newEdge);
			Shell.println("Edge: '" + newEdge.getId()
					+ "' added succesfully to graph '" + graph.getId()
					+ "'!");

			// Change save status of graph
			graph.unsave();
		} catch (Exception ex) {
			Shell.println("Error creating edge... Look into log file for more info.");
			LOGGER.error(Config.ln() + "Edge data error - check input.");
			LOGGER.error(ex.getMessage());
		}
	}

	/**
	 * Prepare specific input to create new graph
	 * 
	 * @param graphId
	 * @param input
	 * @return
	 */
	private static ArrayList<HashMap<String, String>> prepareInputForNewGraph(
			String[] input) {
		ArrayList<HashMap<String, String>> result = new ArrayList<HashMap<String, String>>();
		String graphId = input[2];
		String graphLabel = (input.length == 4) ? input[3] : "\"null\"";

		if (input[2].trim().equals(FILE)) { // reading from file
			String path = input[3];
			try {
				LOGGER.info(Config.ln() + "Prepare input from file '" + path
						+ "'");
				Reader reader = new ReaderFile(path);
				result = reader.read();
			} catch (Exception e) {
				LOGGER.error(Config.ln() + "File '" + path + "' not found.");
				LOGGER.error(e.getMessage());
			}
		} else { // reading from shell directly
			try {
				ReaderShellGraph reader = new ReaderShellGraph(graphId,
						graphLabel);
				result = reader.read();
			} catch (Exception e) {
				LOGGER.error(Config.ln() + "Error reading from shell.");
				LOGGER.error(e.getMessage());
			}
		}

		return result;
	}

	/**
	 * Prepare specific input to add new node to an existing graph
	 * 
	 * @param graphId
	 * @param input
	 * @return
	 */
	private static ArrayList<HashMap<String, String>> prepareInputForNewNode(
			String[] input) {
		ArrayList<HashMap<String, String>> result = new ArrayList<HashMap<String, String>>();
		String graphId = input[2];
		String nodeId = input[3];
		String nodeLabel = (input.length == 5) ? input[4] : "null";

		// Check if the graph we want to add the node exist
		if (Graph.getGraphById(graphId) == null) {
			Shell.println("Graph with ID: '" + graphId + "' does not exist!");
			return null;
		}

		// Try to add the node
		try {
			Reader reader = new ReaderShellNode(nodeId, nodeLabel);
			result = reader.read();
		} catch (Exception e) {
			LOGGER.error(Config.ln() + "Error reading from shell.");
			LOGGER.error(e.getMessage());
		}

		return result;
	}

	/**
	 * Prepare specific input to add new edge to an existing graph
	 * 
	 * @param graphId
	 * @param input
	 * @return
	 */
	private static ArrayList<HashMap<String, String>> prepareInputForNewEdge(
			String[] input) {
		ArrayList<HashMap<String, String>> result = new ArrayList<HashMap<String, String>>();
		String graphId = input[2];
		String edgeId = input[3];
		String nodeFrom = input[4];
		String nodeTo = input[5];
		String edgeLabel = (input.length == 7) ? input[6] : "null";

		// Check if the graph we want to add the edge exist
		if (Graph.getGraphById(graphId) == null) {
			Shell.println("Graph with ID: '" + graphId + "' does not exist!");
			return null;
		}

		// Try to add the edge
		try {
			Reader reader = new ReaderShellEdge(edgeId, nodeFrom, nodeTo,
					edgeLabel);
			result = reader.read();
		} catch (Exception e) {
			LOGGER.error(Config.ln() + "Error reading from shell.");
			LOGGER.error(e.getMessage());
		}

		return result;
	}
}
