package com.kundun.graph.shell;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import org.apache.log4j.Logger;

import com.kundun.graph.controller.Add;
import com.kundun.graph.controller.Display;
import com.kundun.graph.controller.Operation;
import com.kundun.graph.controller.Remove;
import com.kundun.graph.controller.Save;
import com.kundun.graph.main.Config;
import com.kundun.graph.main.GraphRepository;
import com.kundun.graph.model.Graph;

public class Shell {
	
	private final static Logger LOGGER = Logger.getLogger(Shell.class.getName());
	
	public String[] input;
	
	// Reading input tools
	public static BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
	
	private Operation operation;
	
	/** 
	 * Main shell interpreter
	 * @throws IOException 
	 */
	public void run() throws IOException {
		start();
		
		// Main loop, waiting for user input
		while(true) {
			operation = null;
			
			print("> ");
			try {
				input = in.readLine().split("\\s+"); // split by any whitespace
			} catch (IOException e) {
				LOGGER.error(Config.ln() + "Shell input reader exception: ");
				LOGGER.error(e.getMessage());
			} 
			
			switch(input[0].toLowerCase()) {
			
			case "add":
				operation = new Add(input);
				break;
				
			case "rm":
				operation = new Remove(input);
				break;
				
			case "save":
				operation = new Save(input);
				break;
				
			case "display":
				operation = new Display(input);
				break;
				
			case "help":
				displayHelp();
				break;
				
			case "quit":
				finish();
				break;
				
			default:
				println("Invalid command: '" + input[0] + "'. Type 'help' for... help.");
				break;
			}
			// Do the provided command
			if (operation != null) {
				operation.doOperation();
			}
			
		}
	}
	
	////////////////////////////////////////////////////////////

	/**
	 * Quick printer
	 * 
	 * @param s
	 */
	public static void print(String s) {
		System.out.print(s);
	}
	

	/**
	 * Quick printer with newline
	 * 
	 * @param s
	 */
	public static void println(String s) {
		System.out.println(s);
	}
	

	/**
	 * Display startup info
	 */
	private static void start() {
		// Print info
		println("*** Graph Parser, version 0.1 ***");
		println("This is BETA version - please be gentle.");
	}
	

	/**
	 * Check for unsaved graphs, prompt if there are any
	 * @throws IOException 
	 */
	private static void finish() throws IOException {
		if(!thereAreUnsavedGraphs()) {
			end();
		} else {
			println("There are unsaved graphs. If you go now, you'll lose all data.");
			while(true) {
				println("Continue? [yes/no] ");
				println(">");
				String input = in.readLine();
				if (input.toLowerCase().equals("yes")) {
					end();
					break;
				} else if (input.toLowerCase().equals("no")) {
					break;
				} 
			}
		}
	}

	/**
	 * Finish the application and inform the user
	 */
	private static void end() {
		LOGGER.info(Config.ln() + "App closed properly");
		
		println("Application closed");
		System.exit(0);
	}
	
	/**
	 * Display info&help
	 */
	private static void displayHelp() {
		println("List of commands for Graph Parser shell:");
		println("");

		println("add -g -f path");
		println("\tCreate new graph from file located in 'path'.");
		println("");
		
		println("add -g graphId [label]");
		println("\tCreate new, empty graph with id. " +
				"User may provide label.");
		println("");
		
		println("add -n graphId nodeId [label]");
		println("\tCreate new node, add it to existing graph with given graph ID. " +
				"User need to provide node ID, may provide label.");
		println("");
		
		println("add -e graphId edgeId nodeFrom nodeTo [label]");
		println("\tCreate new edge, add it to existing graph with given ID. " +
				"User need to provide two nodes ID, new edge ID and may provide label.");
		println("");
		
		println("rm -g graphId]");
		println("\tRemove graph with given id.");
		println("");
		
		println("rm -n graphId nodeId");
		println("\tRemove node from graph with given id.");
		println("");
		
		println("rm -e graphId edgeId");
		println("\tRemove edge from graph with given id.");
		println("");
		
		println("save -g graphId -f path ");
		println("\tSave a graph to a file located in 'path' or, if it does not exist - " +
				"creates a new one. " +
				"User need to provide graph ID.");
		println("");
		
		println("display graphId ");
		println("\tDisplay a graph with given ID. "+
				"User need to provide graph ID.");
		println("");
		
		println("display -a ");
		println("\tDisplay all graphs currently in application");
		println("");
	}
	
	public static void invalidSyntax() {
		println("Invalid syntax. " + 
				"Type 'help' for... help.");
	}
	
	private static Boolean thereAreUnsavedGraphs() {
		if(!GraphRepository.globalGraphList.isEmpty()) {
			for(Graph g : GraphRepository.globalGraphList) {
				if(!g.isSaved()) {
					return true;
				}
			}
		}
		return false;
	}
}
